function y = fTrancheSpread(FirmNumber, SimulationNumber, rho, Maturity, Tranche6, Tranche5, Tranche4, Tranche3, Tranche2, Tranche1);

lambdaScale = 0.01;
for i=1:FirmNumber;
    Lambdas(i) = lambdaScale;
end

A = randn(SimulationNumber, FirmNumber+1);

for i=1:SimulationNumber
    M(i) = A(i,1);
    for j=1:FirmNumber
        Z(i,j) = A(i,j+1);
        X(i,j) = rho*M(i) + sqrt(1 - rho^2)*Z(i,j);
        U_mat(i,j) = cdf('normal', X(i,j), 0, 1);
    end
end


for cpt=1:SimulationNumber
    for i=1:FirmNumber
        U(i) = U_mat(cpt,i);
    end
    Somme(cpt) = 0;
    for cnt=1:FirmNumber
        if U(cnt) < 1 - exp(-Lambdas(cnt)*Maturity)
            DefaultIndex(cnt) = 1;
        else
            DefaultIndex(cnt) = 0;
        end
        DefaultTime(cnt) = -log(1 - U(cnt)) / Lambdas(cnt);
        Somme(cpt) = Somme(cpt) + DefaultIndex(cnt);
    end
    for i=1:FirmNumber
        for j=1:4*Maturity
            if DefaultTime(i) < 0.25*j
                matrixDefault(i,j) = 1;
            else
                matrixDefault(i,j) = 0;
            end
        end
    end
    defaulttimeQ = zeros(1,4*Maturity);
    for j=1:4*Maturity;
        for i=1:FirmNumber
            defaulttimeQ(j) = defaulttimeQ(j) + matrixDefault(i,j);
        end
    end
    for i=1:4*Maturity
        mat_defaulttimeQ(i,cpt) = defaulttimeQ(i);
    end
end

for i=1:4*Maturity
    for j=1:SimulationNumber
        vector(j) = mat_defaulttimeQ(i,j);
    end
    TS_prob_time = fHistogram(FirmNumber, SimulationNumber, Maturity, vector);
    for k=1:FirmNumber+1;
        TS_prob(k,i) = TS_prob_time(k);
    end
end

for j=1:4*Maturity
    TS_prob_density(1,j) = TS_prob(1,j);
end

for i=2:FirmNumber+1
    for j=1:4*Maturity;
        TS_prob_density(i,j) = TS_prob(i,j) - TS_prob(i-1,j);
    end
end
 
for i=1:FirmNumber+1
    Reduced_Tranche1(i) = max(Tranche1 - (i-1), 0);
    Reduced_Tranche2(i) = max(Tranche2 - max((i-1) - Tranche1, 0), 0);
    Reduced_Tranche3(i) = max(Tranche3 - max((i-1) - (Tranche1 + Tranche2), 0), 0);
    Reduced_Tranche4(i) = max(Tranche4 - max((i-1) - (Tranche1 + Tranche2 + Tranche3), 0), 0);
    Reduced_Tranche5(i) = max(Tranche5 - max((i-1) - (Tranche1 + Tranche2 + Tranche3 + Tranche4), 0), 0);
    Reduced_Tranche6(i) = max(Tranche6 - max((i-1) - (Tranche1 + Tranche2 + Tranche3 + Tranche4 + Tranche5), 0), 0);
end

Rates = xlsread('InterestRates.xls', 'InterestRates', 'B2:U2');
Interest_R = Rates(1:4*Maturity);

for j=1:4*Maturity;
    for i=1:FirmNumber+1
        EV_Tranche1(i, j) = Reduced_Tranche1(i) * TS_prob_density(i, j);
        EV_Tranche2(i, j) = Reduced_Tranche2(i) * TS_prob_density(i, j);
        EV_Tranche3(i, j) = Reduced_Tranche3(i) * TS_prob_density(i, j);
        EV_Tranche4(i, j) = Reduced_Tranche4(i) * TS_prob_density(i, j);
        EV_Tranche5(i, j) = Reduced_Tranche5(i) * TS_prob_density(i, j);
        EV_Tranche6(i, j) = Reduced_Tranche6(i) * TS_prob_density(i, j);
    end
end

DEV_Tranche1 = 0;
DEV_Tranche2 = 0;
DEV_Tranche3 = 0;
DEV_Tranche4 = 0;
DEV_Tranche5 = 0;
DEV_Tranche6 = 0;


for j=1:4*Maturity
    for i=1:FirmNumber+1
        DEV_Tranche1 = DEV_Tranche1 + exp(-Interest_R(j) * 0.25 * j) * EV_Tranche1(i, j);
        DEV_Tranche2 = DEV_Tranche2 + exp(-Interest_R(j) * 0.25 * j) * EV_Tranche2(i, j);
        DEV_Tranche3 = DEV_Tranche3 + exp(-Interest_R(j) * 0.25 * j) * EV_Tranche3(i, j);
        DEV_Tranche4 = DEV_Tranche4 + exp(-Interest_R(j) * 0.25 * j) * EV_Tranche4(i, j);
        DEV_Tranche5 = DEV_Tranche5 + exp(-Interest_R(j) * 0.25 * j) * EV_Tranche5(i, j);
        DEV_Tranche6 = DEV_Tranche6 + exp(-Interest_R(j) * 0.25 * j) * EV_Tranche6(i, j);
    end
end

for j=1:4*Maturity
    for i=1:FirmNumber+1
        EL_Tranche1(i, j) = (Tranche1 - Reduced_Tranche1(i)) * TS_prob_density(i, j);
        EL_Tranche2(i, j) = (Tranche2 - Reduced_Tranche2(i)) * TS_prob_density(i, j);
        EL_Tranche3(i, j) = (Tranche3 - Reduced_Tranche3(i)) * TS_prob_density(i, j);
        EL_Tranche4(i, j) = (Tranche4 - Reduced_Tranche4(i)) * TS_prob_density(i, j);
        EL_Tranche5(i, j) = (Tranche5 - Reduced_Tranche5(i)) * TS_prob_density(i, j);
        EL_Tranche6(i, j) = (Tranche6 - Reduced_Tranche6(i)) * TS_prob_density(i, j);
    end
end

TEL_Tranche1 = zeros(1,4*Maturity);
TEL_Tranche2 = zeros(1,4*Maturity);
TEL_Tranche3 = zeros(1,4*Maturity);
TEL_Tranche4 = zeros(1,4*Maturity);
TEL_Tranche5 = zeros(1,4*Maturity);
TEL_Tranche6 = zeros(1,4*Maturity);

for j=1:4*Maturity
    for i=1:FirmNumber + 1
        TEL_Tranche1(j) = TEL_Tranche1(j) + EL_Tranche1(i, j);
        TEL_Tranche2(j) = TEL_Tranche2(j) + EL_Tranche2(i, j);
        TEL_Tranche3(j) = TEL_Tranche3(j) + EL_Tranche3(i, j);
        TEL_Tranche4(j) = TEL_Tranche4(j) + EL_Tranche4(i, j);
        TEL_Tranche5(j) = TEL_Tranche5(j) + EL_Tranche5(i, j);
        TEL_Tranche6(j) = TEL_Tranche6(j) + EL_Tranche6(i, j);
    end
end
 
DEL_Tranche1 = exp(-Interest_R(1) * 0.25 * 1) * TEL_Tranche1(1);
DEL_Tranche2 = exp(-Interest_R(1) * 0.25 * 1) * TEL_Tranche2(1);
DEL_Tranche3 = exp(-Interest_R(1) * 0.25 * 1) * TEL_Tranche3(1);
DEL_Tranche4 = exp(-Interest_R(1) * 0.25 * 1) * TEL_Tranche4(1);
DEL_Tranche5 = exp(-Interest_R(1) * 0.25 * 1) * TEL_Tranche5(1);
DEL_Tranche6 = exp(-Interest_R(1) * 0.25 * 1) * TEL_Tranche6(1);
 
for i=2:4*Maturity;
    DEL_Tranche1 = DEL_Tranche1 + exp(-Interest_R(i) * 0.25 * i) * (TEL_Tranche1(i) - TEL_Tranche1(i - 1));
    DEL_Tranche2 = DEL_Tranche2 + exp(-Interest_R(i) * 0.25 * i) * (TEL_Tranche2(i) - TEL_Tranche2(i - 1));
    DEL_Tranche3 = DEL_Tranche3 + exp(-Interest_R(i) * 0.25 * i) * (TEL_Tranche3(i) - TEL_Tranche3(i - 1));
    DEL_Tranche4 = DEL_Tranche4 + exp(-Interest_R(i) * 0.25 * i) * (TEL_Tranche4(i) - TEL_Tranche4(i - 1));
    DEL_Tranche5 = DEL_Tranche5 + exp(-Interest_R(i) * 0.25 * i) * (TEL_Tranche5(i) - TEL_Tranche5(i - 1));
    DEL_Tranche6 = DEL_Tranche6 + exp(-Interest_R(i) * 0.25 * i) * (TEL_Tranche6(i) - TEL_Tranche6(i - 1));
end


Par_Yield(1) = DEL_Tranche1 / DEV_Tranche1;
Par_Yield(2) = DEL_Tranche2 / DEV_Tranche2;
Par_Yield(3) = DEL_Tranche3 / DEV_Tranche3;
Par_Yield(4) = DEL_Tranche4 / DEV_Tranche4;
Par_Yield(5) = DEL_Tranche5 / DEV_Tranche5;
Par_Yield(6) = DEL_Tranche6 / DEV_Tranche6;
 
for i=1:6
    Par_spread(i) = Par_Yield(i);
end
 
y = Par_spread;
